@@ -1,11 +1,9 @@
 package org.gcube.applicationsupportlayer.social.mailing;
 
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.Date;
 
 import javax.mail.Message;
-import javax.mail.MessagingException;
 import javax.mail.Multipart;
 import javax.mail.Session;
 import javax.mail.internet.InternetAddress;
@@ -39,24 +37,34 @@ public class EmailPlugin {
 	protected static final int SECONDS2WAIT = 60;
 
 	private static EmailPlugin singleton;
-	
+
 	public static EmailPlugin getInstance() {
 		if (singleton == null)
 			singleton = new EmailPlugin();
 		return singleton;
 	}
-	
+
 	private EmailPlugin() {		
 		new EmailNotificationsConsumer().start();
 	}
-	
+
 	protected static ArrayList<NotificationMail> BUFFER_EMAILS = new ArrayList<NotificationMail>();
-	
-	private static String getHTMLEmail(Notification notification2Save, String userFirstName, String portalURL, String email) {
+
+	private static String getHTMLEmail(Notification notification2Save, String userFirstName, String portalURL, String email, String ... hashtags) {
 
 		String removedMarkup = notification2Save.getDescription().replaceAll("&amp;", "&");
 		removedMarkup = removedMarkup.replaceAll("&gt;", ">");
 		removedMarkup = removedMarkup.replaceAll("&lt;", "<");
+				
+		if (hashtags != null && hashtags.length > 0) {
+			_log.debug("editing hyperlinks for mail client");
+			//notification2Save uri contains the absoulte path to the feed in the correct channel, e.g. /group/ustore_vre?oid=f1637958-34d0-48fc-b5ad-13b1116e389d
+			String pathToVRE = "/group/data-e-infrastructure-gateway?";
+			if (notification2Save.getUri().split("\\?").length > 0) {
+				pathToVRE = notification2Save.getUri().split("\\?")[0];
+			}					
+			removedMarkup = removedMarkup.replace("href=\"?", "href=\""+portalURL + pathToVRE + "?"); //because there is no indication of the portal		
+		}
 
 		String sender = notification2Save.getSenderFullName();
 		if (notification2Save.getType() == NotificationType.DOCUMENT_WORKFLOW_STEP_REQUEST_TASK)
@@ -93,11 +101,18 @@ public class EmailPlugin {
 
 	}
 
-	private static String getTextEmail(Notification notification2Save, String userFirstName, String portalURL, String email) {
+	private static String getTextEmail(Notification notification2Save, String userFirstName, String portalURL, String email, String[] hashtags) {
 
 		String removedMarkup = convertHTML2Text(notification2Save.getDescription());		
-		
 
+		if (hashtags != null && hashtags.length > 0) {
+			for (int i = 0; i < hashtags.length; i++) {		
+				_log.debug("replacing " + hashtags[i]);
+				removedMarkup = removedMarkup.replace(hashtags[i], " " + hashtags[i] + " "); //because removing html cause trimming we put spaces back
+			}
+		}
+		_log.debug("*****removedMarkup=\n" + removedMarkup);
+		
 		String sender = notification2Save.getSenderFullName();
 		if (notification2Save.getType() == NotificationType.DOCUMENT_WORKFLOW_STEP_REQUEST_TASK)
 			sender = "";
@@ -121,9 +136,9 @@ public class EmailPlugin {
 		return body.toString();
 
 	}
-	
 
-	
+
+
 	/**
 	 * Convert html into simple text
 	 * 
@@ -137,8 +152,8 @@ public class EmailPlugin {
 		removedMarkup = removedMarkup.replaceAll("&lt;", "<");
 		String text = removedMarkup;
 		try {
-			 Document document = Jsoup.parse(removedMarkup);
-			 Element body = document.body();
+			Document document = Jsoup.parse(removedMarkup);
+			Element body = document.body();
 			text = buildStringFromNode(body).toString();
 		}
 		catch (Exception e) {
@@ -147,36 +162,36 @@ public class EmailPlugin {
 		}
 		return text;
 	}
-	
+
 	private static StringBuffer buildStringFromNode(Node node) {
-	    StringBuffer buffer = new StringBuffer();
+		StringBuffer buffer = new StringBuffer();
 
-	    if (node instanceof TextNode) {
-	        TextNode textNode = (TextNode) node;
-	        buffer.append(textNode.text().trim());
-	    }
+		if (node instanceof TextNode) {
+			TextNode textNode = (TextNode) node;
+			buffer.append(textNode.text().trim());
+		}
 
-	    for (Node childNode : node.childNodes()) {
-	        buffer.append(buildStringFromNode(childNode));
-	    }
+		for (Node childNode : node.childNodes()) {
+			buffer.append(buildStringFromNode(childNode));
+		}
 
-	    if (node instanceof Element) {
-	        Element element = (Element) node;
-	        String tagName = element.tagName();
-	        if ("p".equals(tagName) || "br".equals(tagName)) {
-	            buffer.append("\n");
-	        }
-	    }
+		if (node instanceof Element) {
+			Element element = (Element) node;
+			String tagName = element.tagName();
+			if ("p".equals(tagName) || "br".equals(tagName)) {
+				buffer.append("\n");
+			}
+		}
 
-	    return buffer;
+		return buffer;
 	}
 
 	private static String getActionLink(Notification notification2Save, String portalURL) {
 		StringBuilder actionLink = new StringBuilder("<a style=\"color:#3B5998; text-decoration:none\" target=\"_blank\" href=\"");
 		return completeActonLinkByNotificationType(notification2Save, actionLink, portalURL);
 	}
-	
-	
+
+
 	/**
 	 * enqueue the message to send
 	 * @param notification2Save
@@ -184,17 +199,12 @@ public class EmailPlugin {
 	 * @param portalName
 	 * @param senderEmail
 	 */
-	public void sendNotification(Notification notification2Save,	String vreName,	String portalName, String senderEmail)  {
-		EmailNotificationProducer thread = new EmailNotificationProducer(new NotificationMail(notification2Save, vreName, portalName, senderEmail));
+	public void sendNotification(Notification notification2Save, String vreName,	String portalName, String senderEmail, String ... hashtags)  {
+		EmailNotificationProducer thread = new EmailNotificationProducer(new NotificationMail(notification2Save, vreName, portalName, senderEmail, hashtags));
 		thread.start();
 	}
-	
-	protected static Message getMessageNotification(
-			Session session, 
-			Notification notification2Save, 
-			String vreName, 
-			String portalName, 
-			String senderEmail) throws Exception {
+
+	protected static Message getMessageNotification(Session session, Notification notification2Save, String vreName, String portalName,	String senderEmail, String ... hashtags) throws Exception {
 
 		UserModel user = null;
 		String portalUrl = null;
@@ -216,13 +226,13 @@ public class EmailPlugin {
 		msg2Return.setFrom(new InternetAddress(senderEmail, portalName));
 		msg2Return.addRecipient(Message.RecipientType.TO, new InternetAddress(email));
 
-		msg2Return.setSubject(getSubjectByNotificationType(notification2Save, portalUrl, vreName, user.getFirstName()));
+		msg2Return.setSubject(getSubjectByNotificationType(notification2Save, portalUrl, vreName, user.getFirstName(), hashtags));
 
 		final MimeBodyPart textPart = new MimeBodyPart();
-		textPart.setContent(getTextEmail(notification2Save, user.getFirstName(), portalUrl, email), "text/plain"); 
+		textPart.setContent(getTextEmail(notification2Save, user.getFirstName(), portalUrl, email, hashtags), "text/plain; charset=UTF-8"); 
 
 		final MimeBodyPart htmlPart = new MimeBodyPart();
-		htmlPart.setContent(getHTMLEmail(notification2Save, user.getFirstName(), portalUrl, email), "text/html; charset=UTF-8");
+		htmlPart.setContent(getHTMLEmail(notification2Save, user.getFirstName(), portalUrl, email, hashtags), "text/html; charset=UTF-8");
 
 		final Multipart mp = new MimeMultipart("alternative");
 		mp.addBodyPart(textPart);
@@ -242,7 +252,7 @@ public class EmailPlugin {
 	 * @param vreName
 	 * @return
 	 */
-	private static String getSubjectByNotificationType(Notification notification2Save, String portalURL, String vreName, String userFirstName) {
+	private static String getSubjectByNotificationType(Notification notification2Save, String portalURL, String vreName, String userFirstName, String ...hashtags) {
 		switch (notification2Save.getType()) {
 		case LIKE:
 			return notification2Save.getSenderFullName()+" favorited your post";
@@ -256,6 +266,12 @@ public class EmailPlugin {
 			return "Removed user in a shared folder";
 		case WP_FOLDER_SHARE:
 			return notification2Save.getSenderFullName()+ " shared a folder with you";
+		case WP_FOLDER_UNSHARE:
+			return notification2Save.getSenderFullName()+ " unshared a folder of yours";
+		case WP_ADMIN_UPGRADE:
+			return notification2Save.getSenderFullName()+ " upgraded you as a folder administrator";
+		case WP_ADMIN_DOWNGRADE:
+			return notification2Save.getSenderFullName()+ " downgraded you as from folder administrator";
 		case WP_ITEM_NEW:
 			return notification2Save.getSenderFullName()+ " added an item in a shared folder";
 		case WP_ITEM_DELETE:
@@ -268,8 +284,12 @@ public class EmailPlugin {
 			return notification2Save.getSenderFullName() + " commented on your post";
 		case MENTION:
 			return notification2Save.getSenderFullName() + " mentioned you";
-		case POST_ALERT:
-			return notification2Save.getSenderFullName() + " shared a news on " + vreName;
+		case POST_ALERT: 
+			String toReturn = notification2Save.getSenderFullName() + " shared a news on " + vreName;
+			if (hashtags != null)
+				for (int i = 0; i < hashtags.length; i++) 
+					toReturn += " " + hashtags[i];				
+			return toReturn;
 		case REQUEST_CONNECTION:
 			return "Connection request";
 		case JOB_COMPLETED_NOK:
